c rearange.f -- Tool to rearange data set using a sample-file 
c 
c***************************************************************************
c                                                                          *
c                         Rowland NMR Toolkit                              *
c                Jeffrey C. Hoch and Blagoje Filpovic                      *
c                                                                          *
c		         Copyright (c) 2007 by                             *
c                University of Connecticut Health Center                   *
c                         263 Farmington Ave.                              *
c                      Farmington, CT 06030-3305                           *
c		          All Rights Reserved			           *
c                                                                          *
c***************************************************************************
c

c
c	rearange schedule-file;l [ADDONE]
c
c rearange a 2D data set along t1 using a sampling schedule file (one integer entry per line)
c the sorted sample schedule is writen to the standard output
c the optional keyword "ADDONE" indicates the 1 is to be added to all sample indices
c the program automatically checks for the presence of a zero in the
c sample schedule and will then add one to all indices
c
	program rearange
	implicit none
	include '../data.cmn'
	include '../access.cmn'
	include '../parse.cmn'
	include '../funcs.cmn'
	character*80 command_line
	integer length
        external main_routine

	call map
	call check_parameters
	call parse(0, command_line, length)

c Carry out the function
	call to_main(main_routine, cbufptr)
	end



c Parse the command-line arguments
	subroutine main_routine(cbuf)
	implicit none
	real cbuf(*)
	include '../data.cmn'
	include '../access.cmn'
	include '../parse.cmn'
	include '../funcs.cmn'
	integer MAXSAMPLES
	parameter (MAXSAMPLES = 32768)
	character*80 line, schedfile
	integer length, lsf
	integer adim, bdim, nbdim, nadim
	integer astride, aioff, bstride, bioff
	integer k, n, ptr, ptr1, j, i
	integer nsamples
	integer aindex(MAXSAMPLES)
	real kbuf(4,MAXSAMPLES),a
        logical ADDONE

c Interpret the arguments and perform the function
	
        if (ndim .ne. 2) then
		write(0,*) 'This program only supports 2 dimensional data'
		return
	endif
        
        ADDONE = .false.
        do i = 1, kl
            if(word(i)(1:6) .eq. "ADDONE") ADDONE = .true.
        enddo
            
  	adim = pdim(1)
	bdim = pdim(2)

        nadim = npts(adim)
        nbdim = npts(bdim)

	if (adim .le. 0 .or. adim .gt. ndim .or.
     1	  bdim .le. 0 .or. bdim .gt. ndim .or.
     2	  abs(adim - bdim) .ne. 1) then
		write(0,*) 'The input dimensions are not consecutive in memory'
		return
	endif

	if (kfl .le. 0) then
		write(0,*) 'Missing schedule file'
		return
	endif
	schedfile = fnam
	lsf = kfl

c Read in the sampling schedule

	if (open_readonly(10, schedfile, lsf) .ne. 0)  return
	k = 0
	n = 0
	do while (.true.)
		call parse(10, line, length)
		n = n + 1
		if (kt .lt. 0)  goto 152
		if (kt .eq. 0)  goto 150
		if (ki .gt. 1) then
			write(0,*) 'Invalid sampling entry on line', n,
     1			  ': ', line(1:length)
		else

			if (k .ge. MAXSAMPLES)  goto 151
			k = k + 1
			aindex(k) = intg(1) 
                        if(aindex(k) .eq. 0) ADDONE = .true.
c			write(*,*) k, aindex(k)
		endif
150		continue
	enddo
151	write(0,*) 'Too many samples: limit is', MAXSAMPLES
	return
152	continue
	close(unit=10)
	nsamples = k
        
	if (nsamples .le. 0) then
		write(0,*) 'No lines listed in the schedule file'
		return
	endif
        
	if (nsamples .ne. nadim) then
	   write(0,*) 'Nlines in schedule file is not equal to npts'
	   return
	endif
        
        write(0,*) 'Number of lines in sampling schedule file:',nsamples
        write(0,*) 'Number of points t1, t2:',nadim,nbdim
	
        astride = istrides(adim)
	aioff = imag_offsets(adim)
	bstride = istrides(bdim)
	bioff = imag_offsets(bdim)
        
      
cc Sorts an arrayarr(1:n) into ascending numerical order, by straight insertion. N is input; 
cc Arr is replaced on output by its sorted rearrangement. 
      
     
cc Pick out each element in turn.     
       do j=2,nadim 
       a=aindex(j)
      
        ptr = 1+(j-1)*astride
          do k = 1, nbdim
		kbuf(1,k) = cbuf(ptr)
		kbuf(2,k) = cbuf(ptr + bioff)
		kbuf(3,k) = cbuf(ptr + aioff)
		kbuf(4,k) = cbuf(ptr + aioff + bioff)
                ptr = ptr + bstride
         enddo

       
cc Lookfor the place to insert it.    
         do i=j-1,1,-1 
           if(aindex(i).le.a) goto 10 
           aindex(i+1) = aindex(i) 
           
           ptr = 1 +(i-1)*astride
           ptr1 = ptr + astride
	  
           do k = 1,nbdim
		
         	cbuf(ptr1) = cbuf(ptr) 
		cbuf(ptr1 + bioff) = cbuf(ptr + bioff)
		cbuf(ptr1 + aioff) = cbuf(ptr + aioff)
		cbuf(ptr1 + aioff + bioff) = cbuf(ptr + aioff + bioff)
	 	                              
                ptr = ptr + bstride
                ptr1 = ptr1 + bstride
	   enddo    
 
         enddo 
         i=0 
cc Insert it.
10    aindex(i+1) = a
          
          ptr= 1+i*astride
          do k = 1, nbdim
		cbuf(ptr) = kbuf(1,k)
		cbuf(ptr + bioff) = kbuf(2,k)
		cbuf(ptr + aioff) = kbuf(3,k)
		cbuf(ptr + aioff + bioff) = kbuf(4,k)
                ptr = ptr + bstride
         enddo 
        
      enddo 
      
      do i = 1, nsamples
        if(ADDONE) then
            write(*,*) aindex(i)+1
        else
            write(*,*) aindex(i)
        endif
        enddo
      end   
        
     
